#include <SoftwareSerial.h> 
#include "instruments.h"
#include "constants.h"

int pwm_a = 3;                          // Left motor
int dir_a = 12;

int pwm_b = 11;                        // Right motor
int dir_b = 13;

byte m_bank = BANK_MELODIC;            // Bank select (melodic, percussion)
byte m_instrument = 0;                 // Instrument IN bank (applies to melodic only)
byte m_note = 0;                       // Note (key, pitch)
byte m_velocity = 127;                 // Note velocity (volume)

byte minKey = NOTE_C4;                 // Beginning pitch for demo
byte maxKey = NOTE_C4 + 3;             // Ending 1 octave higher

// Starting and ending instruments
byte minInstrument = Acoustic_Grand_Piano   ;
byte maxInstrument = Rock_Organ   ;
 
SoftwareSerial MIDI(255, midiIn);      // Rx, Tx (Rx not used)

void setup() {
  setupDrive();
  Serial.begin(9600);
  MIDI.begin(31250);                   // Set up serial comm to MIDI
  resetMidi();                         // Reset MIDI device
}

void loop() {
  playBasic();                         // Continuous loop
}

// Main routine, plays block of pitches (notes) within a selected instrument range
void playBasic() {
  
  ctrlMIDI(CHAN_1, CTRL_VOLUME, m_velocity);      // Set channel volume
  ctrlMIDI(CHAN_1, CTRL_BANK, m_bank);            // Select channel bank
  
   // Iterate through instrument(s) to play
  for(m_instrument = minInstrument ; m_instrument <= maxInstrument ; m_instrument++) {

    // Select instrument within bank
    sendMIDI(CTRL_INSTRUMENT | CHAN_1, m_instrument, 0); 

    // Play through notes
    for (m_note = minKey ; m_note <= maxKey ; m_note++) {
      noteOn(CHAN_1, m_note, m_velocity);
      noteOn(CHAN_1, m_note+4, m_velocity); 
      noteOn(CHAN_1, m_note+7, m_velocity);
      delay(1000);
      
      switch (m_note) {
        case NOTE_C4:
          drive_fwd();
          break;
        case NOTE_C4+1:
          drive_turn();
          break;
        case NOTE_C4+2:
          drive_rev();
          break;
      }
     
      //noteOff(CHAN_1, m_note,  m_velocity); // Turn off just 1 note
      ctrlMIDI(CHAN_1, CTRL_ALL_OFF, 0);       // Turn off channel sounds
      delay(100);
    }
    delay(150);                                  // Pause before next instrument
  }
}

// Tuirn note on (press key)
void noteOn(byte channel, byte note, byte attackVelo) {
  sendMIDI( (NOTE_ON | channel), note, attackVelo);
}

// Turn note off (release key)
void noteOff(byte channel, byte note, byte releaseVelo) {
  sendMIDI( (NOTE_OFF | channel), note, releaseVelo);
}

// Set controller value with channel
void ctrlMIDI(byte channel, byte data1, byte data2) {
  sendMIDI( (CONTROLLER | channel), data1, data2);
}

// Send bytes to MIDI device
void sendMIDI(byte cmd, byte data1, byte data2) {
  MIDI.write(cmd);
  MIDI.write(data1);
  // Pass only if valid 2nd byte
  if( (cmd & 0xF0) <= 0xB0 ||  (cmd & 0xF0) == 0xE0 )
    MIDI.write(data2);
}

// Reset MIDI device
void resetMidi() {
  pinMode(resetPin, OUTPUT);
  digitalWrite(resetPin, LOW);
  delay(100);
  digitalWrite(resetPin, HIGH);
  delay(100);
}

void setupDrive() {
  pinMode(pwm_a, OUTPUT);  //Set control pins to be outputs
  pinMode(pwm_b, OUTPUT);
  pinMode(dir_a, OUTPUT);
  pinMode(dir_b, OUTPUT);
}

void drive_fwd() {
  digitalWrite(dir_a, LOW);
  digitalWrite(dir_b, LOW);
  analogWrite(pwm_a, 255); 
  analogWrite(pwm_b, 255);
}

void drive_turn() {
  digitalWrite(dir_a, LOW);
  digitalWrite(dir_b, HIGH);
  setPWM(pwm_a, 70);
  setPWM(pwm_b, 70);
}

void drive_rev() {
  digitalWrite(dir_a, HIGH);
  digitalWrite(dir_b, HIGH);
  analogWrite(pwm_a, 255); 
  analogWrite(pwm_b, 255);
}

void setPWM(int pin, int val) {
  analogWrite(pin, map(val, 0, 100, 0, 255));
}
